@@ -1,7 +1,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003-2018 by The University of Queensland
+# Copyright (c) 2003-2020 by The University of Queensland
 # http://www.uq.edu.au
 #
 # Primary Business: Queensland, Australia
@@ -11,6 +11,7 @@
 # Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 # Development 2012-2013 by School of Earth Sciences
 # Development from 2014 by Centre for Geoscience Computing (GeoComp)
+# Development from 2019 by School of Earth and Environmental Sciences
 #
 ##############################################################################
 
@@ -18,7 +19,7 @@
 
 from __future__ import print_function, division
 
-__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
+__copyright__="""Copyright (c) 2003-2020 by The University of Queensland
 http://www.uq.edu.au
 Primary Business: Queensland, Australia"""
 __license__="""Licensed under the Apache License, version 2.0
@@ -68,57 +69,139 @@ def _zoom(phi, gradphi, phiargs, alpha_lo, alpha_hi, phi_lo, phi_hi, c1, c2,
     Helper function for `line_search` below which tries to tighten the range
     alpha_lo...alpha_hi. See Chapter 3 of 'Numerical Optimization' by
     J. Nocedal for an explanation.
+    interpolation options are linear, quadratic, cubic or Newton interpolation.
     """
-    def quadinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi):
-        if alpha_lo > alpha_hi:
-            alpha_hi,phi_hi=alpha_lo,phi_lo
-        denom=2*phi_hi-2*phi0-2*alpha_hi*gphi0
-        if denom == 0:
-            alpha=0.5*(alpha_lo+alpha_hi)
-        else:
-            alpha=(-alpha_hi**2.0*gphi0)/denom
-            if alpha < alpha_lo or alpha > alpha_hi:
-                alpha=0.5*(alpha_lo+alpha_hi)
+
+    def linearinterpolate(alpha_lo,alpha_hi,old_alpha):
+        alpha = 0.5*(alpha_lo+alpha_hi)
         return alpha
 
-    def cubicinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi,old_alpha,old_phi):
-        if alpha_lo > alpha_hi:
-            alpha_hi,phi_hi=alpha_lo,phi_lo
-        if alpha_lo==0:
-            alpha_1,phi_1,alpha_2,phi_2=alpha_hi,phi_hi,old_alpha,old_phi
-        else:
-            alpha_1,phi_1,alpha_2,phi_2=alpha_lo,phi_lo,alpha_hi,phi_hi
-        if alpha_1 == 0 or alpha_2 == 0 or alpha_1 == alpha_2:
-            return quadinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi)    
-        a1s,a2s,a1c,a2c=alpha_1**2,alpha_2**2,alpha_1**3,alpha_2**3
-        a=(-a1s*phi_2+a2s*phi_1-a2s*phi0+(a1s*alpha_2-alpha_1*a2s)*gphi0)/(a1c*a2s-a1s*a2c)
-        b=-(-a1c*phi_2+a2c*phi_1-a2c*phi0+(a1c*alpha_2-alpha_1*a2c)*gphi0)/(a1c*a2s-a1s*a2c)
-        deter=b**2.0-3.0*a*gphi0
-        if deter<0:
-            return quadinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi)
-        A,B=(-b/(3.0*a)),sqrt(deter)/(3.0*a)
-        alpha=A-B+2*B*(6*a*(A+B)+2*b>0)
-        if alpha < alpha_lo or alpha > alpha_hi:
-            return quadinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi)
+    def quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi):
+        if old_alpha is None:
+            return linearinterpolate(alpha_lo,alpha_hi,old_alpha)
+        denom=2.0*(old_phi-phi0-gphi0*old_alpha)
+        if denom == 0:
+            return linearinterpolate(alpha_lo,alpha_hi,old_alpha)
+        alpha=-gphi0*old_alpha*old_alpha/denom
         if np.abs(alpha-old_alpha) < tol_df or np.abs(alpha) < tol_sm*np.abs(old_alpha):
-            alpha=0.5*(alpha_lo+alpha_hi)
+            alpha=0.5*old_alpha
+        if abs(alpha) < 1e-8:
+            return linearinterpolate(alpha_lo,alpha_hi,old_alpha)
         return alpha
 
+    def cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi):
+        if very_old_alpha is None:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        a0s,a1s=very_old_alpha*very_old_alpha,old_alpha*old_alpha
+        denom=a0s*a1s*(old_alpha-very_old_alpha)
+        if denom==0:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        a0c,a1c=a0s*very_old_alpha,a1s*old_alpha
+        tmpA=old_phi-phi0-gphi0*old_alpha 
+        tmpB=very_old_phi-phi0-gphi0*very_old_alpha
+        a=( a0s*tmpA-a1s*tmpB)/denom
+        b=(-a0c*tmpA+a1c*tmpB)/denom
+        deter=b*b-3.0*a*gphi0
+        if deter<0:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        alpha=(-b+sqrt(deter))/(3.0*a)
+        if np.abs(alpha-old_alpha) < tol_df or np.abs(alpha) < tol_sm*np.abs(old_alpha):
+            alpha=0.5*old_alpha
+        if abs(alpha) < 1e-8:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        return alpha
+
+    def newtoninterpolate(alpha_data,phi_data,old_alpha,old_phi):
+        # Interpolates using a polynomial of increasing order
+        # The coefficients of the interpolated polynomial are found using the Newton method
+        if very_old_alpha is None:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        if old_alpha in alpha_data:
+            return quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
+        alpha_data.append(old_alpha)
+        phi_data.append(old_phi)
+        coefs=newton_poly(alpha_data,phi_data)
+        alpha=newtonroot(coefs,alpha_data,old_alpha)
+        if alpha < alpha_lo or alpha > alpha_hi: # Root is outside the domain
+            zoomlogger.debug("NOTE: Newton interpolation converged on a root outside of [alpha_lo,alpha_hi]. Falling back on cubic interpolation")
+            return cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi)
+        if np.abs(alpha-old_alpha) < tol_df or np.abs(alpha) < tol_sm*np.abs(old_alpha):
+            alpha=0.5*old_alpha
+        if abs(alpha) <= 0:
+            zoomlogger.debug("NOTE: Newton interpolation returned alpha <= 0. Falling back on cubic interpolation")
+            return cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi)
+        return alpha
+
+    def newton_poly(alpha_data,phi_data):
+        # Returns the coefficients of the newton form polynomial of phi(alpha)
+        # for the points alpha_data and function values phi_data
+        m=len(alpha_data)
+        x=np.copy(alpha_data)
+        a=np.copy(phi_data)
+        for k in range(1,m):
+            a[k:m]=(a[k:m]-a[k-1])/(x[k:m]-x[k-1])
+        return a
+
+    def newtonroot(coefs,alpha_data,startingGuess):
+        # Solves for the root of a polynomial using the newton method
+        dfcoefs=list(range(1,len(coefs)))
+        for i in range(0,len(coefs)-1):
+            dfcoefs[i]*=coefs[i+1]
+        tol=1e-6
+        error=100
+        maxiterations=100
+        point=startingGuess
+        counter=0
+        for i in range(0,maxiterations):
+            counter+=1
+            product=1
+            numer=coefs[0]
+            denom=dfcoefs[0]
+            for i in range(1, len(coefs)):
+                product*=(point-alpha_data[i-1])
+                numer+=product*coefs[i]
+                if i < len(coefs)-1:
+                    denom+=product*dfcoefs[i]
+            if denom == 0:
+                zoomlogger.debug("NOTE: The Newton solver failed (denom==0). Falling back on cubic interpolation")                
+                return cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi)
+            newpoint=float(point-numer/denom)
+            error=abs(newpoint-point)
+            if error < tol:
+                zoomlogger.debug("Newton interpolation converged in %d iterations. Got alpha = %f" % (counter, newpoint))
+                return newpoint
+            else:
+                point=newpoint
+        # zoomlogger.debug("NOTE: Newton interpolation failed to converge (exceeded max iterations). Falling back on cubic interpolation" % error)
+        return cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi)
+
     i=0
-    if interpolationOrder == 3:
-        old_alpha=0.5*(alpha_lo+alpha_hi)
-        old_phi=phi(old_alpha)
+
+    # Setup
+    old_alpha=None
+    old_phi=None
+    very_old_alpha=None
+    very_old_phi=None
+    alpha_data=[0.0]
+    phi_data=[phi0]
+
     while i<=IMAX:
         if interpolationOrder == 1:
-            alpha=0.5*(alpha_lo+alpha_hi)
+            alpha=linearinterpolate(alpha_lo,alpha_hi,old_alpha)
         elif interpolationOrder == 2:
-            alpha=quadinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi)
+            alpha=quadinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi)
         elif interpolationOrder == 3:
-            alpha=cubicinterpolate(alpha_lo,phi_lo,alpha_hi,phi_hi,old_alpha,old_phi)
+            alpha=cubicinterpolate(alpha_lo,alpha_hi,old_alpha,old_phi,very_old_alpha,very_old_phi)
+        elif interpolationOrder == 4:
+            alpha=newtoninterpolate(alpha_data,phi_data,old_alpha,old_phi)
+        else:
+            raise TypeError("Invalid interpolation order")
+
         phiargs(alpha)
         phi_a=phi(alpha)
         zoomlogger.debug("iteration %d, alpha=%e, phi(alpha)=%e"%(i,alpha,phi_a))
         if phi_a > phi0+c1*alpha*gphi0 or phi_a >= phi_lo:
+            very_old_alpha,very_old_phi=old_alpha,old_phi
             old_alpha,old_phi=alpha_hi,phi_hi
             alpha_hi,phi_hi=alpha,phi_a
         else:
@@ -128,6 +211,7 @@ def _zoom(phi, gradphi, phiargs, alpha_lo, alpha_hi, phi_lo, phi_hi, c1, c2,
                 break
             if gphi_a*(alpha_hi-alpha_lo) >= 0:
                 alpha_hi = alpha_lo
+            very_old_alpha,very_old_phi=old_alpha,old_phi
             old_alpha,old_phi=alpha_lo,phi_lo
             alpha_lo,phi_lo=alpha,phi_a
         if not alpha_hi > alpha_lo:
@@ -153,10 +237,10 @@ def line_search(f, x, p, g_Jx, Jx, args_x, alpha=1.0, alpha_truncationax=50.0,
     :param c1: value for Armijo condition (see reference)
     :param c2: value for curvature condition (see reference)
     :param interpolationOrder: the order of the interpolation used (1, 2 or 3)
-    :param tol_df: if using the cubic interpolation, the new value of alpha must 
-    :               differ from the previous value by at least this much
-    :param tol_sm: if using the cubic interpolation, the new value of alpha must
-    :               not be less than tol_sm*(alpha_{i-1})
+    :param tol_df: the interpolated value of alpha, alpha_i, must differ from the previous
+    :             value by at least this much i.e. abs(alpha_i-alpha_{i-1}) < tol_df
+    :param tol_sm: the interpolated value of alpha must not be less than tol_sm
+    :             i.e. abs(alpha_i) < tol_sm*abs(alpha_{i-1})
     :param IMAX: maximum number of iterations to perform
     """
     # this stores the latest gradf(x+a*p) which is returned
@@ -203,7 +287,7 @@ def line_search(f, x, p, g_Jx, Jx, args_x, alpha=1.0, alpha_truncationax=50.0,
     old_phi_a=phi0
     phi_a=phi0
     i=1
-    if (interpolationOrder != 1) and (interpolationOrder != 2) and (interpolationOrder != 3):
+    if (interpolationOrder != 1) and (interpolationOrder != 2) and (interpolationOrder != 3) and (interpolationOrder != 4):
         lslogger.debug("WARNING invalid interpolation order. Setting interpolationOrder = 1")
         interpolationOrder=1
     #alpha=2.*alpha
@@ -237,7 +321,7 @@ def line_search(f, x, p, g_Jx, Jx, args_x, alpha=1.0, alpha_truncationax=50.0,
 ##############################################################################
 class AbstractMinimizer(object):
     """
-    Base class for function minimization methods.
+    Base class for function minimization methods.   
     """
 
     def __init__(self, J=None, m_tol=1e-4, J_tol=None, imax=300):
@@ -246,6 +330,10 @@ class AbstractMinimizer(object):
 
         :param J: the cost function to be minimized
         :type J: `CostFunction`
+        :param m_tol: terminate interations when relative change of the level set 
+                      function is less than or equal m_tol
+        :type m_tol: float
+ 
         """
         self.setCostFunction(J)
         self.setMaxIterations(imax)
@@ -253,7 +341,7 @@ class AbstractMinimizer(object):
         self._callback = None
         self.logger = logging.getLogger('inv.%s'%self.__class__.__name__)
         self.setTolerance(m_tol=m_tol, J_tol=J_tol)
-        self.interpolationOrder=2
+        self.interpolationOrder=1
 
     def setCostFunction(self, J):
         """
@@ -346,7 +434,8 @@ class MinimizerLBFGS(AbstractMinimizer):
     """
     Minimizer that uses the limited-memory Broyden-Fletcher-Goldfarb-Shanno
     method.
-    """
+    See Chapter 6 of 'Numerical Optimization' by J. Nocedal for an explanation.
+    """ 
 
     # History size
     _truncation = 30
@@ -362,11 +451,66 @@ class MinimizerLBFGS(AbstractMinimizer):
 
     # maximum number of zoom steps in line search
     _max_zoom_steps = 50
+
+         
+    # tolerance for the interpolated value of alpha in line search must not be less than tol_sm
+    # i.e. abs(alpha_i) < tol_sm*abs(alpha_{i-1})
+    _tol_sm = 1e-5
+    
+    # the interpolated value of alpha, alpha_i in line search, must differ from the previous 
+    # value by at least this much i.e. abs(alpha_i-alpha_{i-1}) < tol_df    
+    _tol_df = 1e-6
     
     def getOptions(self):
-        return {'truncation':self._truncation,'initialHessian':self._initial_H, 'restart':self._restart, 'max_linesearch_steps' : self._max_linesearch_steps, 'max_zoom_steps' : self._max_zoom_steps}
+        """
+        returns a dictionary of LBFGS options 
+        rtype: dictionary with keys 'truncation', 'initialHessian', 'restart', 'max_linesearch_steps', 'max_zoom_steps'
+        'interpolationOrder', 'tol_df', 'tol_sm' 
+        """
+        return {'truncation':self._truncation,'initialHessian':self._initial_H, 'restart':self._restart,
+                 'max_linesearch_steps' : self._max_linesearch_steps, 'max_zoom_steps' : self._max_zoom_steps, 
+                'interpolationOrder': self.interpolationOrder,'tol_df': self._tol_df, 'tol.sm': self._tol_sm}
+
+
 
     def setOptions(self, **opts):
+        """
+        setOptions for LBFGS.  use       solver.setOptions( key = value)
+        :key truncation: sets the number of previous LBFGS iterations to keep
+        :type truncation : int
+        :default truncation: 30
+        :key initialHessian: 1 if initial Hessian is given
+        :type initialHessian: 1 or 0    
+        :default initialHessian: 1       
+        :key restart: restart after this many iteration steps
+        :type restart: int
+        :default restart: 60
+        :key max_linesearch_steps: maximum number of line search iterations
+        :type max_linesearch_steps: int
+        :default max_linesearch_steps: 25  
+        :key max_zoom_steps: maximum number of zoom iterations
+        :type max_zoom_steps: int
+        :default max_zoom_steps: 60
+        :key interpolationOrder: order of the interpolation used for line search 
+        :type interpolationOrder: 1,2,3 or 4
+        :default interpolationOrder: 1      
+        :key tol_df: interpolated value of alpha, alpha_i, must differ
+                   : from the previous value by at least this much 
+                   : abs(alpha_i-alpha_{i-1}) < tol_df  (line search)
+        :type tol_df: float
+        :default tol_df: 1e-6
+        :key tol_sm: interpolated value of alpha must not be less than tol_sm
+                   : abs(alpha_i) < tol_sm*abs(alpha_{i-1})
+        :type tol_sm: float
+        :default tol_sm: 1e-5     
+        
+            Example of usage::
+              cf=DerivedCostFunction()
+              solver=MinimizerLBFGS(J=cf, m_tol = 1e-5, J_tol = 1e-5, imax=300)
+              solver.setOptions(truncation=20, tol_df =1e-7)
+              solver.run(initial_m)
+              result=solver.getResult()
+        """
         self.logger.debug("Setting options: %s"%(str(opts)))
         for o in opts:
             if o=='historySize' or o=='truncation':
@@ -598,6 +742,31 @@ class MinimizerBFGS(AbstractMinimizer):
         return {'initialHessian':self._initial_H}
 
     def setOptions(self, **opts):
+        """
+        setOptions for LBFGS.      use   solver.setOptions( key = value)
+        :key initialHessian: 1 if initial Hessian is given
+        :type initialHessian: 1 or 0    
+        :default initialHessian: 1       
+        :key interpolationOrder: order of the interpolation used for line search 
+        :type interpolationOrder: 1,2,3 or 4
+        :default interpolationOrder: 1      
+        :key tol_df: interpolated value of alpha, alpha_i, must differ
+                   : from the previous value by at least this much 
+                   : abs(alpha_i-alpha_{i-1}) < tol_df  (line search)
+        :type tol_df: float
+        :default tol_df: 1e-6
+        :key tol_sm: interpolated value of alpha must not be less than tol_sm
+                   : abs(alpha_i) < tol_sm*abs(alpha_{i-1})
+        :type tol_sm: float
+        :default tol_sm: 1e-5     
+        
+            Example of usage::
+              cf=DerivedCostFunction()
+              solver=MinimizerBFGS(J=cf, m_tol = 1e-5, J_tol = 1e-5, imax=300)
+              solver.setOptions(tol_df =1e-7)
+              solver.run(initial_m)
+              result=solver.getResult()
+        """
         for o in opts:
             if o=='initialHessian':
                 self._initial_H=opts[o]
