@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #ifndef __DUDLEY_DOMAIN_H__
@@ -620,6 +621,15 @@ public:
     */
     virtual escript::Data getX() const;
 
+    /**
+     \brief returns locations in the FEM nodes as a numpy ndarray
+    */
+#ifdef ESYS_HAVE_BOOST_NUMPY
+    boost::python::numpy::ndarray getNumpyX() const;
+
+    boost::python::numpy::ndarray getConnectivityInfo() const;
+#endif
+
     /**
      \brief returns boundary normals at the quadrature point on the face
             elements
