@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #include "DataVector.h"
@@ -207,8 +208,8 @@ namespace escript {
    {
       using namespace std;
       
-      ESYS_ASSERT(data != 0, "Error - data is null");
-      ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
+      // ESYS_ASSERT(data != 0, "Error - data is null");
+      // ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
 
       switch (getRank(shape)) {
       case 0:
@@ -262,8 +263,8 @@ namespace escript {
       using namespace std;
 
       
-      ESYS_ASSERT(data != 0, "Error - data is null");
-      ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
+      // ESYS_ASSERT(data != 0, "Error - data is null");
+      // ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
 
       switch (getRank(shape)) {
       case 0:
@@ -315,8 +316,8 @@ namespace escript {
    {
       using namespace std;
 
-      ESYS_ASSERT(data != 0, "Error - data is null");
-      ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
+      // ESYS_ASSERT(data != 0, "Error - data is null");
+      // ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
 
       switch (getRank(shape)) {
       case 0:
@@ -369,8 +370,8 @@ namespace escript {
       
       using namespace std;
       
-      ESYS_ASSERT(data != 0, "Error - data is null");
-      ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
+      // ESYS_ASSERT(data != 0, "Error - data is null");
+      // ESYS_ASSERT(data.size() > 0,"Error - Data object is empty.");
 
       switch (getRank(shape)) {
       case 0:
