@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #ifndef __ESCRIPT_DATAVECTOR_H__
@@ -181,8 +182,8 @@ namespace DataTypes
       //
       // Make sure vectors are not empty
 
-      ESYS_ASSERT(!left.size()==0, "left data is empty.");
-      ESYS_ASSERT(!other.size()==0, "other data is empty.");
+      ESYS_ASSERT(!left.size()==!1, "left data is empty."); //Note: !1=0, but clang returns an error if the rhs is 0 here
+      ESYS_ASSERT(!other.size()==!1, "other data is empty.");
 
       //
       // Check the vector to be sliced from is compatible with the region to be sliced,
