@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #ifndef __ESCRIPT_DATAMATHS_H__
@@ -687,8 +688,11 @@ Note that vector in this context refers to a data vector storing datapoints not
               const DataTypes::ShapeType& evShape,
               typename DataTypes::CplxVectorType::size_type evOffset)
   {
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-variable"
    typename DataTypes::CplxVectorType::ElementType in00,in10,in20,in01,in11,in21,in02,in12,in22;
    typename DataTypes::CplxVectorType::ElementType ev0,ev1,ev2;
+#pragma clang diagnostic pop
    int s=inShape[0];
    if (s==1) {
       in00=in[inOffset+DataTypes::getRelIndex(inShape,0,0)];
