@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #include "SolverOptions.h"
@@ -38,15 +39,8 @@ SolverBuddy::SolverBuddy() :
     method(SO_DEFAULT),
     preconditioner(SO_PRECONDITIONER_JACOBI),
     ode_solver(SO_ODESOLVER_LINEAR_CRANK_NICOLSON),
-    smoother(SO_PRECONDITIONER_GAUSS_SEIDEL),
     reordering(SO_REORDERING_DEFAULT),
-    coarsening(SO_DEFAULT),
-    amg_interpolation_method(SO_INTERPOLATION_DIRECT),
-    level_max(100),
-    coarsening_threshold(0.25),
     sweeps(1),
-    pre_sweeps(1),
-    post_sweeps(1),
     tolerance(1e-8),
     absolute_tolerance(0.),
     inner_tolerance(0.9),
@@ -58,19 +52,18 @@ SolverBuddy::SolverBuddy() :
     restart(0),
     is_complex(false),
     symmetric(false),
+    hermitian(false),
     verbose(false),
     adapt_inner_tolerance(true),
     accept_convergence_failure(false),
-    min_coarse_matrix_size(500),
     relaxation(0.3),
     use_local_preconditioner(false),
-    min_sparsity(0.05),
     refinements(2),
-    coarse_refinements(2),
-    use_panel(true),
-    diagonal_dominance_threshold(0.5),
-    cycle_type(1)
+    dim(2),
+    using_default_solver_method(false)
 {
+    // setPackage(SO_DEFAULT);
+    // setSolverMethod(SO_DEFAULT);
     resetDiagnostics(true);
 }
 
@@ -83,72 +76,27 @@ std::string SolverBuddy::getSummary() const
 {
     std::stringstream out;
     out << "Solver Package = " << getName(getPackage()) << std::endl
-        << "Solver target = " << getName(getSolverTarget()) << std::endl
         << "Verbosity = " << isVerbose() << std::endl
-        << "Accept failed convergence = " << acceptConvergenceFailure()
-        << std::endl
+        << "Accept failed convergence = " << acceptConvergenceFailure() << std::endl
         << "Relative tolerance = " << getTolerance() << std::endl
         << "Absolute tolerance = " << getAbsoluteTolerance() << std::endl
         << "Symmetric problem = " << isSymmetric() << std::endl
+        << "Hermitian problem = " << isHermitian() << std::endl
         << "Maximum number of iteration steps = " << getIterMax() << std::endl
         << "Inner tolerance = " << getInnerTolerance() << std::endl
         << "Adapt innner tolerance = " << adaptInnerTolerance() << std::endl;
 
     if (getPackage() == SO_DEFAULT || getPackage() == SO_PACKAGE_PASO ||
-            getPackage() == SO_PACKAGE_CUSP ||
             getPackage() == SO_PACKAGE_TRILINOS) {
         out << "Solver method = " << getName(getSolverMethod()) << std::endl;
         if (getSolverMethod() == SO_METHOD_GMRES) {
             out << "Truncation  = " << getTruncation() << std::endl
                 << "Restart  = " << getRestart() << std::endl;
-        } else if (getSolverMethod() == SO_PRECONDITIONER_AMG) {
-            out << "Number of pre / post sweeps = " << getNumPreSweeps()
-                << " / " << getNumPostSweeps() << ", " << getNumSweeps()
-                << std::endl
-                << "Maximum number of levels = " << getLevelMax() << std::endl
-                << "Coarsening threshold = " << getCoarseningThreshold()
-                << std::endl
-                << "Coarsening method = " << getName(getCoarsening())
-                << std::endl;
         }
         out << "Preconditioner = " << getName(getPreconditioner()) << std::endl
             << "Apply preconditioner locally = " << useLocalPreconditioner()
             << std::endl;
         switch (getPreconditioner()) {
-            case SO_PRECONDITIONER_AMG:
-                out << "Maximum number of levels = " << getLevelMax()
-                    << std::endl
-                    << "Coarsening threshold = " << getCoarseningThreshold()
-                    << std::endl
-                    << "Minimal sparsity on coarsest level = "
-                    << getMinCoarseMatrixSparsity() << std::endl
-                    << "Smoother = " << getName(getSmoother()) << std::endl
-                    << "Minimum size of the coarsest level matrix = "
-                    << getMinCoarseMatrixSize() << std::endl
-                    << "Number of pre / post sweeps = " << getNumPreSweeps()
-                    << " / " << getNumPostSweeps() << ", " << getNumSweeps()
-                    << std::endl
-                    << "Number of refinement steps in coarsest level solver = "
-                    << getNumCoarseMatrixRefinements() << std::endl
-                    << "Use node panel = " << usePanel() << std::endl
-                    << "Interpolation = " << getName(getAMGInterpolation())
-                    << std::endl
-                    << "Threshold for diagonal dominant rows = "
-                    << getDiagonalDominanceThreshold() << std::endl;
-                break;
-            case SO_PRECONDITIONER_AMLI:
-                out << "Maximum number of levels = " << getLevelMax()
-                    << std::endl
-                    << "Coarsening method = " << getName(getCoarsening())
-                    << std::endl
-                    << "Coarsening threshold = " << getMinCoarseMatrixSize()
-                    << std::endl
-                    << "Minimum size of the coarsest level matrix = "
-                    << getCoarseningThreshold() << std::endl
-                    << "Number of pre / post sweeps = " << getNumPreSweeps()
-                    << " / " << getNumPostSweeps() << ", " << getNumSweeps()
-                    << std::endl;
-                break;
             case SO_PRECONDITIONER_GAUSS_SEIDEL:
                 out << "Number of sweeps = " << getNumSweeps() << std::endl;
                 break;
@@ -175,7 +123,6 @@ const char* SolverBuddy::getName(int key) const
         case SO_TARGET_CPU: return "CPU";
         case SO_TARGET_GPU: return "GPU";
 
-        case SO_PACKAGE_CUSP: return "CUSP";
         case SO_PACKAGE_MKL: return "MKL";
         case SO_PACKAGE_PASO: return "PASO";
         case SO_PACKAGE_TRILINOS: return "TRILINOS";
@@ -203,7 +150,6 @@ const char* SolverBuddy::getName(int key) const
         case SO_METHOD_TFQMR: return "TFQMR";
 
         case SO_PRECONDITIONER_AMG: return "AMG";
-        case SO_PRECONDITIONER_AMLI: return "AMLI";
         case SO_PRECONDITIONER_GAUSS_SEIDEL: return "GAUSS_SEIDEL";
         case SO_PRECONDITIONER_ILU0: return "ILU0";
         case SO_PRECONDITIONER_ILUT: return "ILUT";
@@ -221,16 +167,6 @@ const char* SolverBuddy::getName(int key) const
             return "CLASSIC_INTERPOLATION_WITH_FF";
         case SO_INTERPOLATION_DIRECT: return "DIRECT_INTERPOLATION";
 
-        case SO_COARSENING_AGGREGATION: return "AGGREGATION_COARSENING";
-        case SO_COARSENING_CIJP: return "CIJP_COARSENING";
-        case SO_COARSENING_CIJP_FIXED_RANDOM:
-            return "CIJP_FIXED_RANDOM_COARSENING";
-        case SO_COARSENING_FALGOUT: return "FALGOUT_COARSENING";
-        case SO_COARSENING_HMIS: return "HMIS_COARSENING";
-        case SO_COARSENING_PMIS: return "PMIS_COARSENING";
-        case SO_COARSENING_RUGE_STUEBEN: return "RUGE_STUEBEN_COARSENING";
-        case SO_COARSENING_STANDARD: return "STANDARD_COARSENING";
-        case SO_COARSENING_YAIR_SHAPIRA: return "YAIR_SHAPIRA_COARSENING";
 
         case SO_REORDERING_DEFAULT: return "DEFAULT_REORDERING";
         case SO_REORDERING_MINIMUM_FILL_IN: return "MINIMUM_FILL_IN";
@@ -371,7 +307,6 @@ double SolverBuddy::getDiagnostics(const std::string name) const
 {
     if (name == "num_iter") return num_iter;
     else if (name == "cum_num_iter") return cum_num_iter;
-    else if (name == "num_level") return num_level;
     else if (name == "num_inner_iter") return num_inner_iter;
     else if (name == "cum_num_inner_iter") return cum_num_inner_iter;
     else if (name == "time") return time;
@@ -385,9 +320,6 @@ double SolverBuddy::getDiagnostics(const std::string name) const
     else if (name == "preconditioner_size") return preconditioner_size;
     else if (name == "time_step_backtracking_used")
         return  time_step_backtracking_used;
-    else if (name == "coarse_level_sparsity") return coarse_level_sparsity;
-    else if (name == "num_coarse_unknowns") return num_coarse_unknowns;
-
     throw ValueError(std::string("unknown diagnostic item: ") + name);
 }
 
@@ -396,62 +328,18 @@ bool SolverBuddy::hasConverged() const
     return converged;
 }
 
-void SolverBuddy::setCoarsening(int method)
-{
-    SolverOptions meth = static_cast<SolverOptions>(method);
-    switch (meth) {
-        case SO_DEFAULT:
-        case SO_COARSENING_AGGREGATION:
-        case SO_COARSENING_CIJP:
-        case SO_COARSENING_CIJP_FIXED_RANDOM:
-        case SO_COARSENING_FALGOUT:
-        case SO_COARSENING_HMIS:
-        case SO_COARSENING_PMIS:
-        case SO_COARSENING_RUGE_STUEBEN:
-        case SO_COARSENING_STANDARD:
-        case SO_COARSENING_YAIR_SHAPIRA:
-            coarsening = meth;
-            break;
-        default:
-            throw ValueError("unknown coarsening method");
-    }
-}
-
-SolverOptions SolverBuddy::getCoarsening() const
-{
-    return coarsening;
-}
-
-void SolverBuddy::setMinCoarseMatrixSize(int size)
-{
-    if (size < 0) {
-        throw ValueError("minimum size of the coarsest level "
-                                     "matrix must be non-negative.");
-    }
-    min_coarse_matrix_size = size;
-}
-
-int SolverBuddy::getMinCoarseMatrixSize() const
-{
-    return min_coarse_matrix_size;
-}
-
 void SolverBuddy::setPreconditioner(int precon)
 {
     SolverOptions preconditioner = static_cast<SolverOptions>(precon);
     switch(preconditioner) {
         case SO_PRECONDITIONER_AMG:
-/*
-#ifdef ESYS_MPI
-            throw ValueError("AMG preconditioner is not supported in MPI builds");
-            break;
+#ifndef ESYS_HAVE_TRILINOS
+        throw ValueError("escript was not compiled with Trilinos enabled");
 #endif
-*/
-        case SO_PRECONDITIONER_AMLI:
         case SO_PRECONDITIONER_GAUSS_SEIDEL:
+        case SO_PRECONDITIONER_JACOBI: // This is the default preconditioner in ifpack2
         case SO_PRECONDITIONER_ILU0:
         case SO_PRECONDITIONER_ILUT:
-        case SO_PRECONDITIONER_JACOBI:
         case SO_PRECONDITIONER_NONE:
         case SO_PRECONDITIONER_REC_ILU:
         case SO_PRECONDITIONER_RILU:
@@ -467,26 +355,19 @@ SolverOptions SolverBuddy::getPreconditioner() const
     return preconditioner;
 }
 
-void SolverBuddy::setSmoother(int s)
-{
-    SolverOptions smoother = static_cast<SolverOptions>(s);
-    if (smoother != SO_PRECONDITIONER_JACOBI &&
-            smoother != SO_PRECONDITIONER_GAUSS_SEIDEL) {
-        throw ValueError("unknown smoother");
-    }
-    this->smoother = smoother;
-}
-
-SolverOptions SolverBuddy::getSmoother() const
-{
-    return smoother;
-}
-
 void SolverBuddy::setSolverMethod(int method)
 {
     SolverOptions meth = static_cast<SolverOptions>(method);
+
+//     bool havePASODirect = false;
+//     using_default_solver_method=false;
+// #if defined(ESYS_HAVE_PASO) && (defined(ESYS_HAVE_MKL) || defined(ESYS_HAVE_UMFPACK))
+//     havePASODirect = true;
+// #endif
+
     switch(meth) {
         case SO_DEFAULT:
+        case SO_METHOD_ITERATIVE:
         case SO_METHOD_BICGSTAB:
         case SO_METHOD_CGLS:
         case SO_METHOD_CGS:
@@ -494,7 +375,6 @@ void SolverBuddy::setSolverMethod(int method)
         case SO_METHOD_CR:
         case SO_METHOD_GMRES:
         case SO_METHOD_HRZ_LUMPING:
-        case SO_METHOD_ITERATIVE:
         case SO_METHOD_LSQR:
         case SO_METHOD_MINRES:
         case SO_METHOD_NONLINEAR_GMRES:
@@ -531,36 +411,59 @@ SolverOptions SolverBuddy::getSolverMethod() const
     return method;
 }
 
-void SolverBuddy::setSolverTarget(int target)
-{
-    SolverOptions targ = static_cast<SolverOptions>(target);
-    switch (targ) {
-        case SO_TARGET_CPU:
-        case SO_TARGET_GPU:
-            this->target = targ;
-            break;
-        default:
-            throw ValueError("unknown solver target");
-    }
-}
-
-SolverOptions SolverBuddy::getSolverTarget() const
-{
-    return target;
-}
-
 void SolverBuddy::setPackage(int package)
 {
     SolverOptions pack = static_cast<SolverOptions>(package);
     switch (pack) {
         case SO_DEFAULT:
-        case SO_PACKAGE_CUSP:
-        case SO_PACKAGE_MKL:
-        case SO_PACKAGE_PASO:
-        case SO_PACKAGE_TRILINOS:
-        case SO_PACKAGE_UMFPACK:
-            this->package = pack;
+        // Default to trilinos if it is available, else use PASO
+        // This should always work because escript cannot be compiled
+        // unless at least one of these is available
+#ifdef ESYS_HAVE_TRILINOS
+            this->package = SO_PACKAGE_TRILINOS;
+            setSolverMethod(getSolverMethod());
             break;
+#else
+            this->package = SO_PACKAGE_PASO;
+            setSolverMethod(getSolverMethod());
+            break;
+#endif
+        // Set to PASO iff escript was compiled with PASO
+        case SO_PACKAGE_PASO:
+#ifdef ESYS_HAVE_PASO
+            this->package = SO_PACKAGE_PASO;
+            setSolverMethod(getSolverMethod());
+            break;
+#else
+            throw ValueError("escript was not compiled with PASO enabled");
+#endif
+        // Set to Trilinos iff escript was compiled with Trilinos
+        case SO_PACKAGE_TRILINOS:
+#ifdef ESYS_HAVE_TRILINOS
+            this->package = SO_PACKAGE_TRILINOS;
+            setSolverMethod(getSolverMethod());
+            break;
+#else
+            throw ValueError("escript was not compiled with Trilinos enabled");
+#endif
+        // Set to MKL iff escript was compiled with MKL
+        case SO_PACKAGE_MKL:
+#ifdef ESYS_HAVE_MKL
+            this->package = SO_PACKAGE_MKL;
+            setSolverMethod(getSolverMethod());
+            break;
+#else
+            throw ValueError("escript was not compiled with MKL enabled");
+#endif
+        // Set to Umfpack iff escript was compiled with Umfpack
+        case SO_PACKAGE_UMFPACK:
+#ifdef ESYS_HAVE_UMFPACK
+            this->package = SO_PACKAGE_UMFPACK;
+            setSolverMethod(getSolverMethod());
+            break;
+#else
+            throw ValueError("escript was not compiled with UMFPACK enabled");
+#endif
         default:
             throw ValueError("unknown solver package");
     }
@@ -612,18 +515,6 @@ int SolverBuddy::_getRestartForC() const
     return r;
 }
 
-void SolverBuddy::setDiagonalDominanceThreshold(double value)
-{
-    if (value < 0. || value > 1.)
-        throw ValueError("Diagonal dominance threshold must be between 0 and 1.");
-    diagonal_dominance_threshold = value;
-}
-
-double SolverBuddy::getDiagonalDominanceThreshold() const
-{
-    return diagonal_dominance_threshold;
-}
-
 void SolverBuddy::setTruncation(int truncation)
 {
     if (truncation < 1)
@@ -660,40 +551,6 @@ int SolverBuddy::getIterMax() const
     return iter_max;
 }
 
-void SolverBuddy::setLevelMax(int level_max)
-{
-    if (level_max < 0)
-        throw ValueError("maximum number of coarsening levels must be non-negative.");
-    this->level_max = level_max;
-}
-
-int SolverBuddy::getLevelMax() const
-{
-    return level_max;
-}
-
-void SolverBuddy::setCycleType(int cycle_type)
-{
-    this->cycle_type = cycle_type;
-}
-
-int SolverBuddy::getCycleType() const
-{
-    return cycle_type;
-}
-
-void SolverBuddy::setCoarseningThreshold(double theta)
-{
-    if (theta < 0. || theta > 1.)
-        throw ValueError("threshold must be between 0 and 1.");
-    coarsening_threshold = theta;
-}
-
-double SolverBuddy::getCoarseningThreshold() const
-{
-    return coarsening_threshold;
-}
-
 void SolverBuddy::setNumSweeps(int sweeps)
 {
     if (sweeps < 1)
@@ -706,30 +563,6 @@ int SolverBuddy::getNumSweeps() const
     return sweeps;
 }
 
-void SolverBuddy::setNumPreSweeps(int sweeps)
-{
-    if (sweeps < 1)
-        throw ValueError("number of pre-sweeps must be positive.");
-    pre_sweeps = sweeps;
-}
-
-int SolverBuddy::getNumPreSweeps() const
-{
-    return pre_sweeps;
-}
-
-void SolverBuddy::setNumPostSweeps(int sweeps)
-{
-    if (sweeps < 1)
-       throw ValueError("number of post-sweeps must be positive.");
-    post_sweeps = sweeps;
-}
-
-int SolverBuddy::getNumPostSweeps() const
-{
-    return post_sweeps;
-}
-
 void SolverBuddy::setTolerance(double rtol)
 {
     if (rtol < 0. || rtol > 1.)
@@ -835,6 +668,29 @@ void SolverBuddy::setSymmetry(bool flag)
         setSymmetryOff();
 }
 
+bool SolverBuddy::isHermitian() const
+{
+    return hermitian;
+}
+
+void SolverBuddy::setHermitianOn()
+{
+    hermitian = true;
+}
+
+void SolverBuddy::setHermitianOff()
+{
+    hermitian = false;
+}
+
+void SolverBuddy::setHermitian(bool flag)
+{
+    if (flag)
+        setHermitianOn();
+    else
+        setHermitianOff();
+}
+
 bool SolverBuddy::isVerbose() const
 {
     return verbose;
@@ -927,18 +783,6 @@ void SolverBuddy::setLocalPreconditioner(bool use)
         setLocalPreconditionerOff();
 }
 
-void SolverBuddy::setMinCoarseMatrixSparsity(double sparsity)
-{
-    if (sparsity < 0. || sparsity > 1.)
-        throw ValueError("sparsity must be between 0 and 1.");
-    min_sparsity = sparsity;
-}
-
-double SolverBuddy::getMinCoarseMatrixSparsity() const
-{
-    return min_sparsity;
-}
-
 void SolverBuddy::setNumRefinements(int refinements)
 {
     if (refinements < 0)
@@ -951,60 +795,6 @@ int SolverBuddy::getNumRefinements() const
     return refinements;
 }
 
-void SolverBuddy::setNumCoarseMatrixRefinements(int refinements)
-{
-    if (refinements < 0)
-        throw ValueError("number of coarse matrix refinements must be non-negative.");
-    coarse_refinements = refinements;
-}
-
-int SolverBuddy::getNumCoarseMatrixRefinements() const
-{
-    return coarse_refinements;
-}
-
-bool SolverBuddy::usePanel() const
-{
-    return use_panel;
-}
-
-void SolverBuddy::setUsePanelOn()
-{
-    use_panel = true;
-}
-
-void SolverBuddy::setUsePanelOff()
-{
-    use_panel = false;
-}
-
-void SolverBuddy::setUsePanel(bool use)
-{
-    if (use)
-        setUsePanelOn();
-    else
-        setUsePanelOff();
-}
-
-void SolverBuddy::setAMGInterpolation(int method)
-{
-    SolverOptions meth = static_cast<SolverOptions>(method);
-    switch (meth) {
-        case SO_INTERPOLATION_CLASSIC:
-        case SO_INTERPOLATION_CLASSIC_WITH_FF_COUPLING:
-        case SO_INTERPOLATION_DIRECT:
-            amg_interpolation_method = meth;
-            break;
-        default:
-            throw ValueError("unknown AMG interpolation method");
-    }
-}
-
-SolverOptions SolverBuddy::getAMGInterpolation() const
-{
-    return amg_interpolation_method;
-}
-
 void SolverBuddy::setODESolver(int method)
 {
     SolverOptions ode = static_cast<SolverOptions>(method);
@@ -1037,5 +827,21 @@ bp::dict SolverBuddy::getTrilinosParameters() const
     return trilinosParams;
 }
 
-} // namespace escript
+void SolverBuddy::setDim(int dim)
+{
+    if (dim != 2 && dim != 3)
+        throw ValueError("Dimension must be either 2 or 3.");
+    this->dim = dim;
+}
 
+int SolverBuddy::getDim()
+{
+    return dim;
+}
+
+bool SolverBuddy::using_default_method() const
+{
+    return using_default_solver_method;
+}
+
+} // namespace escript
