@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #include "FinleyDomain.h"
@@ -56,22 +57,6 @@ struct ElementInfo {
     int tag;
 };
 
-inline bool is_node_string(const char* line)
-{
-    if (!line)
-        return false;
-    return !strncmp(line, "$NOD", 4) || !strncmp(line, "$NOE", 4)
-            || !strncmp(line, "$Nodes", 6);
-}
-
-inline bool is_endnode_string(const char* line)
-{
-    if (!line)
-        return false;
-    return !strncmp(line, "$ENDNOD", 7) || !strncmp(line, "$ENDNOE", 7)
-            || !strncmp(line, "$EndNodes", 9);
-}
-
 inline bool get_line(std::vector<char>& line, FILE* file)
 {
     int capacity = 1024;
@@ -370,7 +355,10 @@ int getElementsMaster(escript::JMPI& mpiInfo, FinleyDomain* dom,
     if (!get_line(line, fileHandle))
         errorFlag = EARLY_EOF;
     int scan_ret, numEntityBlocks;
-    if(version >= 4.0){
+    if (version >= 4.1) {
+        int minNodeTag, maxNodeTag;
+        scan_ret = sscanf(&line[0], "%d %d %d %d", &numEntityBlocks, &totalNumElements, &minNodeTag, &maxNodeTag);
+    } else if (version == 4.0){
         scan_ret = sscanf(&line[0], "%d %d", &numEntityBlocks, &totalNumElements);
     } else {
         scan_ret = sscanf(&line[0], "%d", &totalNumElements);
@@ -419,7 +407,7 @@ int getElementsMaster(escript::JMPI& mpiInfo, FinleyDomain* dom,
                 errorFlag = EARLY_EOF;
 
             int entityTag, entityDim, numElements, gmsh_type;
-            scan_ret = sscanf(&line[0], "%d %d %d %d", &entityTag, &entityDim, &gmsh_type, &numElements);
+            scan_ret = sscanf(&line[0], "%d %d %d %d", &entityDim, &entityTag, &gmsh_type, &numElements);
 
             // Loop over elements in each entity block
             for (index_t e = 0; e < numElements; e++, count++) {
@@ -431,10 +419,9 @@ int getElementsMaster(escript::JMPI& mpiInfo, FinleyDomain* dom,
                 struct ElementInfo element = {NoRef, 0, 0, &vertices[count*MAX_numNodes_gmsh], 0};
 
                 get_line(line, fileHandle);
-                getSingleElementMSH4(fileHandle, numDim, version, element, 
+                getSingleElementMSH4(fileHandle, numDim, version, element,
                                         errorMsg, filename, useMacroElements, gmsh_type, &line[0]);
-                
-                
+                element.tag = entityTag;
                 elementType[count] = element.type;
                 id[count] = element.id;
                 tag[count] = element.tag;
@@ -513,7 +500,7 @@ int getElementsMaster(escript::JMPI& mpiInfo, FinleyDomain* dom,
         #endif
             }
         }
-        
+
     } else { // Version < 4.0
         for (index_t e = 0, count = 0; e < totalNumElements; e++, count++) {
             if (cpuId >= mpiInfo->size-1) {
@@ -708,36 +695,77 @@ int gather_nodes(FILE* f, std::map<int,int>& tags, std::string& errorMsg,
                  int dim, double version, const std::string& filename)
 {
     int numNodes=0;
+    int numEntityBlocks;
     std::vector<char> line;
     if (!get_line(line, f))
         return EARLY_EOF;
     int scan_ret;
-    if(version >= 4.0){
+    if(version >= 4.1){
+        int minNodes, maxNodes;
+        scan_ret = sscanf(&line[0], "%d %d %d %d", &numEntityBlocks, &numNodes, &minNodes, &maxNodes);
+    } else if (version == 4.0){
         int tmp;
         scan_ret = sscanf(&line[0], "%d %d", &tmp, &numNodes);
         numNodes += tmp;
     } else {
-        scan_ret = sscanf(&line[0], "%d", &numNodes);    
+        scan_ret = sscanf(&line[0], "%d", &numNodes);
     }
-    
+
     if (scan_ret == EOF)
         return EARLY_EOF;
-    for (int node = 0; node < numNodes; node++) {
-        int tmp = 0;
-        if (!get_line(line, f))
-            return EARLY_EOF;
-        int scan_ret = sscanf(&line[0], "%d", &tmp);
-        if (scan_ret == EOF) {
-            return EARLY_EOF;
-        } else if (scan_ret != 1) {
-            errorMsg = "malformed meshfile";
-            return THROW_ERROR;
+    if(version >= 4.1){
+        for (int entity = 0; entity < numEntityBlocks; entity++) {
+            int entityDim, entityTag, parametric, numNodesInBlock;
+            if (!get_line(line, f))
+                return EARLY_EOF;
+            scan_ret = sscanf(&line[0], "%d %d %d %d", &entityDim, &entityTag, &parametric, &numNodesInBlock);
+
+            if (parametric == 1){
+                errorMsg = "eScript does not supprot nodefiles with parametric coordinates.";
+                return THROW_ERROR;
+            }
+
+            // Tag information
+            for (int nodes = 0; nodes < numNodesInBlock; nodes++){
+                int tag;
+                if (!get_line(line, f))
+                    return EARLY_EOF;
+                scan_ret = sscanf(&line[0], "%d", &tag);
+                if (scan_ret != 1){
+                    errorMsg = "malformed meshfile (broken node section)!";
+                    return THROW_ERROR;
+                }
+            }
+            // Node coordinate information
+            for (int nodes = 0; nodes < numNodesInBlock; nodes++){
+                if (!get_line(line, f))
+                    return EARLY_EOF;
+                float x, y, z;
+                scan_ret = sscanf(&line[0], "%f %f %f", &x, &y, &z);
+                if (scan_ret != 3){
+                    errorMsg = "malformed meshfile (broken node section)!";
+                    return THROW_ERROR;
+                }
+            }
+        }
+    } else {
+        for (int node = 0; node < numNodes; node++) {
+            int tmp = 0;
+            if (!get_line(line, f))
+                return EARLY_EOF;
+            scan_ret = sscanf(&line[0], "%d", &tmp);
+            if (scan_ret == EOF) {
+                return EARLY_EOF;
+            } else if (scan_ret != 1) {
+                errorMsg = "malformed meshfile (broken node section)!";
+                return THROW_ERROR;
+            }
+            tags[tmp] = -1;
         }
-        tags[tmp] = -1;
     }
     if (!get_line(line, f))
         return EARLY_EOF;
-    if (!is_endnode_string(&line[0])) {
+    if(!(!strncmp(&line[0], "$ENDNOD", 7) || !strncmp(&line[0], "$ENDNOE", 7) || !strncmp(&line[0], "$EndNodes", 9))){
         std::stringstream ss;
         ss << "readGmsh: malformed mesh file. Expected '$EndNodes', got '"
             << &line[0] << "'";
@@ -757,22 +785,37 @@ int gather_nodes(FILE* f, std::map<int,int>& tags, std::string& errorMsg,
     int numBlocks = -1;
     if (!get_line(line, f))
         return EARLY_EOF;
-    if(version >= 4.0){
+    if (version >= 4.1){
+        int minElements, maxElements;
+        scan_ret = sscanf(&line[0], "%d %d %d %d\n", &numBlocks, &numElements, &minElements, &maxElements);
+    } else if (version == 4.0){
         scan_ret = sscanf(&line[0], "%d %d\n", &numBlocks, &numElements);
     } else {
         scan_ret = sscanf(&line[0], "%d\n", &numElements);
     }
     if (scan_ret == EOF) {
         return EARLY_EOF;
-    } else if ((scan_ret != 1 && version < 4.0) || (scan_ret != 2 && version >= 4.0)) {
-        errorMsg = "readGmsh: malformed mesh file";
+    } else if ((scan_ret != 1 && version < 4.0) || (scan_ret != 2 && version == 4.0) || (scan_ret != 4 && version >= 4.1)) {
+        errorMsg = "readGmsh: malformed mesh file ($Elements section contains incorrect header information)";
         return THROW_ERROR;
     }
     struct ElementInfo e;
     std::vector<int> v(MAX_numNodes_gmsh, -1);
     e.vertex = &v[0];
 
-    if(version >= 4.0){
+    if(version >= 4.1){
+        int entityTag, entityDim, gmsh_type;
+        for(int blocks = 0; blocks < numBlocks; blocks++){
+            if (!get_line(line, f))
+                return EARLY_EOF;
+            scan_ret = sscanf(&line[0], "%d %d %d %d", &entityDim, &entityTag, &gmsh_type, &numElements); //Note that entityDim and entityTag are switched in version 4.0
+            for(int elementNumber = 0; elementNumber < numElements; elementNumber++){
+                if (!get_line(line, f))
+                    return EARLY_EOF;
+                getSingleElementMSH4(f, dim, version, e, errorMsg, filename, false, gmsh_type, &line[0]);
+            }
+        }
+    } else if (version == 4.0){
         for(int x = 0; x < numBlocks; x++){
             int entityTag, entityDim, gmsh_type;
             if (!get_line(line, f))
@@ -781,7 +824,7 @@ int gather_nodes(FILE* f, std::map<int,int>& tags, std::string& errorMsg,
             for(int i = 0; i < numElements; i++){
                 if (!get_line(line, f))
                     return EARLY_EOF;
-                getSingleElementMSH4(f, dim, version, e, errorMsg, filename, false, gmsh_type, &line[0]);    
+                getSingleElementMSH4(f, dim, version, e, errorMsg, filename, false, gmsh_type, &line[0]);
             }
         }
     } else {
@@ -806,7 +849,7 @@ int gather_nodes(FILE* f, std::map<int,int>& tags, std::string& errorMsg,
 }
 
 int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
-                   int numDim, std::string& errorMsg, std::map<int, int>& tags, int version)
+                   int numDim, std::string& errorMsg, std::map<int, int>& tags, double version)
 {
     int errorFlag = 0;
     std::vector<char> line;
@@ -815,12 +858,15 @@ int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
 
     int numNodes = 0, numBlocks = 0;
     int scan_ret;
-    if(version >= 4.0){
+    if(version >= 4.1){
+        int minNodeTag, maxNodeTag;
+        scan_ret = sscanf(&line[0], "%d %d %d %d", &numBlocks, &numNodes, &minNodeTag, &maxNodeTag);
+    }else if(version == 4.0){
         scan_ret = sscanf(&line[0], "%d %d", &numBlocks, &numNodes);
     } else {
         scan_ret = sscanf(&line[0], "%d", &numNodes);
     }
-    
+
     SSCANF_CHECK(scan_ret);
 #ifdef ESYS_MPI
     // Broadcast numNodes if there are multiple mpi procs
@@ -841,70 +887,57 @@ int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
 
         if (!errorFlag) {
             //read in chunksize nodes
-            if(version >= 4.0){
+            if(version == 4.1){
+                int nodeCounter = -1;
                 for (int x = 0; x < numBlocks; x++) {
                     std::vector<char> line;
                     if (!get_line(line, fileHandle))
                         errorFlag = EARLY_EOF;
 
-                    if (is_endnode_string(&line[0])) {
+                    if (!strncmp(&line[0], "$ENDNOD", 7) || !strncmp(&line[0], "$ENDNOE", 7) || !strncmp(&line[0], "$EndNodes", 9)) {
                         errorMsg = "readGmsh: found end node string while still reading nodes!";
                         errorFlag = THROW_ERROR;
                         break;
                     }
 
                     int entityTag, entityDim, parametric, numDataPoints;
-                    scan_ret = sscanf(&line[0], "%d %d %d %d\n", &entityTag, &entityDim, &parametric, &numDataPoints);
+                    scan_ret = sscanf(&line[0], "%d %d %d %d\n", &entityDim, &entityTag, &parametric, &numDataPoints);
+
+                    if (parametric == 1){
+                        errorMsg = "eScript does not supprot nodefiles with parametric coordinates.";
+                        return THROW_ERROR;
+                    }
+
+                    // Tag information. At the moment this information is discarded.
+                    int temp = nodeCounter;
+                    for (int nodes = 0; nodes < numDataPoints; nodes++){
+                        if (!get_line(line, fileHandle))
+                            return EARLY_EOF;
+                        temp++;
+                        scan_ret = sscanf(&line[0], "%d", &tempInts[temp]);
+                        if (scan_ret != 1){
+                            errorMsg = "malformed meshfile (broken node section)!";
+                            return THROW_ERROR;
+                        }
+                    }
 
                     for(int j = 0; j < numDataPoints; j++){
                         // Get the next line
                         if (!get_line(line, fileHandle))
-                        errorFlag = EARLY_EOF;
+                            return EARLY_EOF;
 
                         // Read the information
-                        if(!parametric){
-                            scan_ret = sscanf(&line[0], "%d %le %le %le\n", &tempInts[(totalNodes+j)], 
-                                        &tempCoords[0+(totalNodes+j)*numDim], 
-                                        &tempCoords[1+(totalNodes+j)*numDim], 
-                                        &tempCoords[2+(totalNodes+j)*numDim]);
+                        nodeCounter++;
+                        if (1 == numDim) {
+                            scan_ret = sscanf(&line[0], "%le\n", &tempCoords[0+nodeCounter*numDim]);
+                            SSCANF_CHECK(scan_ret);
+                        } else if (2 == numDim) {
+                            scan_ret = sscanf(&line[0], "%le %le\n", &tempCoords[0+nodeCounter*numDim], &tempCoords[1+nodeCounter*numDim]);
+                            SSCANF_CHECK(scan_ret);
+                        } else if (3 == numDim) {
+                            scan_ret = sscanf(&line[0], "%le %le %le\n", &tempCoords[0+nodeCounter*numDim], &tempCoords[1+nodeCounter*numDim], &tempCoords[2+nodeCounter*numDim]);
                             SSCANF_CHECK(scan_ret);
-                        } else {
-                            double u = 0., v = 0.; // These are the parametric coordinates. At the moment the code throws this information away
-                            switch(entityDim){
-                                case 0:
-                                    scan_ret = sscanf(&line[0], "%d %le %le %le\n", &tempInts[(totalNodes+j)], 
-                                        &tempCoords[0+(totalNodes+j)*numDim], 
-                                        &tempCoords[1+(totalNodes+j)*numDim], 
-                                        &tempCoords[2+(totalNodes+j)*numDim]);
-                                    SSCANF_CHECK(scan_ret);
-                                    break;
-                                case 1:
-                                    scan_ret = sscanf(&line[0], "%d %le %le %le %le\n", &tempInts[(totalNodes+j)], 
-                                        &tempCoords[0+(totalNodes+j)*numDim], 
-                                        &tempCoords[1+(totalNodes+j)*numDim], 
-                                        &tempCoords[2+(totalNodes+j)*numDim], &u);
-                                    SSCANF_CHECK(scan_ret);
-                                    break;
-                                case 2:
-                                    scan_ret = sscanf(&line[0], "%d %le %le %le %le %le\n", &tempInts[(totalNodes+j)], 
-                                        &tempCoords[0+(totalNodes+j)*numDim], 
-                                        &tempCoords[1+(totalNodes+j)*numDim], 
-                                        &tempCoords[2+(totalNodes+j)*numDim], &u, &v);
-                                    SSCANF_CHECK(scan_ret);
-                                    break;
-                                case 3:
-                                    scan_ret = sscanf(&line[0], "%d %le %le %le\n", &tempInts[(totalNodes+j)], 
-                                        &tempCoords[0+(totalNodes+j)*numDim], 
-                                        &tempCoords[1+(totalNodes+j)*numDim], 
-                                        &tempCoords[2+(totalNodes+j)*numDim]);
-                                    SSCANF_CHECK(scan_ret);
-                                    break;
-                                default:
-                                    errorMsg = "readGmsh: information in $nodes is in the wrong format!";
-                                    errorFlag = THROW_ERROR;
-                            }
                         }
-
                     }
                     totalNodes += numDataPoints;
                 }
@@ -918,7 +951,59 @@ int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
                     errorFlag = THROW_ERROR;
                     break;
                 }
-                
+
+            } else if(version == 4.0){
+                int nodeCounter = -1;
+                for (int x = 0; x < numBlocks; x++) {
+                    std::vector<char> line;
+                    if (!get_line(line, fileHandle))
+                        errorFlag = EARLY_EOF;
+
+                    if (!strncmp(&line[0], "$ENDNOD", 7) || !strncmp(&line[0], "$ENDNOE", 7) || !strncmp(&line[0], "$EndNodes", 9)) {
+                        errorMsg = "readGmsh: found end node string while still reading nodes!";
+                        errorFlag = THROW_ERROR;
+                        break;
+                    }
+
+                    int entityTag, entityDim, parametric, numNodes;
+                    scan_ret = sscanf(&line[0], "%d %d %d %d\n", &entityTag, &entityDim, &parametric, &numNodes);
+                    
+                    if (parametric == 1){
+                        errorMsg = "eScript does not supprot nodefiles with parametric coordinates.";
+                        return THROW_ERROR;
+                    }
+
+                    for(int j = 0; j < numNodes; j++){
+                        // Get the next line
+                        if (!get_line(line, fileHandle))
+                            return EARLY_EOF;
+
+                        // Read the information
+                        nodeCounter++;
+                        if (1 == numDim) {
+                            scan_ret = sscanf(&line[0], "%d %le\n", &tempInts[nodeCounter], &tempCoords[0+nodeCounter*numDim]);
+                            SSCANF_CHECK(scan_ret);
+                        } else if (2 == numDim) {
+                            scan_ret = sscanf(&line[0], "%d %le %le\n", &tempInts[nodeCounter], &tempCoords[0+nodeCounter*numDim], &tempCoords[1+nodeCounter*numDim]);
+                            SSCANF_CHECK(scan_ret);
+                        } else if (3 == numDim) {
+                            scan_ret = sscanf(&line[0], "%d %le %le %le\n", &tempInts[nodeCounter], &tempCoords[0+nodeCounter*numDim], &tempCoords[1+nodeCounter*numDim], &tempCoords[2+nodeCounter*numDim]);
+                            SSCANF_CHECK(scan_ret);
+                        }
+                    }
+                    totalNodes += numNodes;
+                }
+
+                // Possible error (if nodes file is malformed)
+                if (totalNodes > numNodes) {
+                    std::stringstream ss;
+                    ss << "readGmsh: too many nodes (" << totalNodes << " < "
+                        << numNodes << ")";
+                    errorMsg = ss.str();
+                    errorFlag = THROW_ERROR;
+                    break;
+                }
+
             } else { // Not msh version 4.0 or higher
                 for (int chunkNodes = 0; chunkNodes < chunkSize; chunkNodes++) {
                     if (totalNodes > numNodes) {
@@ -933,7 +1018,7 @@ int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
                     if (!get_line(line, fileHandle))
                         errorFlag = EARLY_EOF;
 
-                    if (is_endnode_string(&line[0])) {
+                    if (!strncmp(&line[0], "$ENDNOD", 7) || !strncmp(&line[0], "$ENDNOE", 7) || !strncmp(&line[0], "$EndNodes", 9)) {
                         errorMsg = "readGmsh: found end node string while still reading nodes!";
                         errorFlag = THROW_ERROR;
                         break;
@@ -979,8 +1064,8 @@ int getNodesMaster(escript::JMPI& mpiInfo, FinleyDomain* dom, FILE* fileHandle,
     NodeFile* nodes = dom->getNodes();
     nodes->allocTable(chunkSize);
 
-#pragma omp parallel for schedule(static)
-    for (index_t i = 0; i < chunkSize; i++) {
+//#pragma omp parallel for schedule(static)
+    for(index_t i = 0; i < chunkSize; i++) {
         nodes->Id[i] = tempInts[i];
         nodes->globalDegreesOfFreedom[i] = tempInts[i];
         int tag = tags[tempInts[i]];
@@ -1018,7 +1103,7 @@ int get_next_state(FILE *f, bool nodesRead, bool elementsRead, int *logicFlag) {
 
     if (!strncmp(&line[1], "MeshFormat", 10)) {
         *logicFlag = 1;
-    } else if (is_node_string(&line[0])) {
+    } else if (!strncmp(&line[0], "$NOD", 4) || !strncmp(&line[0], "$NOE", 4) || !strncmp(&line[0], "$Nodes", 6)) {
         *logicFlag = 2;
     } else if (!strncmp(&line[1], "ELM", 3) || !strncmp(&line[1], "Elements", 8)) {
         *logicFlag = 3;
@@ -1251,7 +1336,7 @@ int getNodesSlave(escript::JMPI& mpiInfo, FinleyDomain* dom, int numDim,
     NodeFile* nodes = dom->getNodes();
     nodes->allocTable(chunkNodes);
 
-#pragma omp parallel for schedule(static)
+//#pragma omp parallel for schedule(static)
     for (index_t i = 0; i < chunkNodes; i++) {
         nodes->Id[i] = tempInts[i];
         nodes->globalDegreesOfFreedom[i] = tempInts[i];
@@ -1506,4 +1591,3 @@ escript::Domain_ptr FinleyDomain::readGmsh(escript::JMPI mpiInfo,
 }
 
 } // namespace finley
-
