@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2018 by The University of Queensland
+* Copyright (c) 2003-2020 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,8 +10,9 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014 by Centre for Geoscience Computing (GeoComp)
-*
+* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
+* Development from 2019 by School of Earth and Environmental Sciences
+**
 *****************************************************************************/
 
 #include <ripley/RipleyDomain.h>
@@ -22,9 +23,6 @@
 #include <escript/index.h>
 #include <escript/SolverOptions.h>
 
-#ifdef ESYS_HAVE_CUDA
-#include <ripley/RipleySystemMatrix.h>
-#endif
 #ifdef ESYS_HAVE_TRILINOS
 #include <trilinoswrap/TrilinosMatrixAdapter.h>
 #endif
@@ -546,7 +544,7 @@ void RipleyDomain::interpolateOnDomain(escript::Data& target,
                                 copy(src, src+numComp, target.getSampleDataRW(m_diracPoints[i].node));
                             }
                         }
-                
+
                 }
                 break;
             default:
@@ -560,6 +558,18 @@ escript::Data RipleyDomain::getX() const
     return escript::continuousFunction(*this).getX();
 }
 
+#ifdef ESYS_HAVE_BOOST_NUMPY
+boost::python::numpy::ndarray RipleyDomain::getNumpyX() const
+{
+    return escript::continuousFunction(*this).getNumpyX();
+}
+
+boost::python::numpy::ndarray RipleyDomain::getConnectivityInfo() const
+{
+    throw RipleyException("This feature is currently not supported by Ripley.");
+}
+#endif
+
 escript::Data RipleyDomain::getNormal() const
 {
     return escript::functionOnBoundary(*this).getNormal();
@@ -669,6 +679,11 @@ void RipleyDomain::setToIntegralsWorker(std::vector<Scalar>& integrals,
                 assembleIntegrate(integrals, funcArg);
             }
             break;
+        case Points:
+            {
+                assembleIntegrate(integrals, arg);
+            }
+            break;
         case Elements:
         case ReducedElements:
         case FaceElements:
@@ -864,35 +879,6 @@ int RipleyDomain::getSystemMatrixTypeId(const bp::object& options) const
     bool isDirect = escript::isDirectSolver(method);
 #endif
 
-    // use CUSP for single rank and supported solvers+preconditioners if CUDA
-    // is available, PASO or Trilinos otherwise
-#ifdef ESYS_HAVE_CUDA
-    if (package == escript::SO_DEFAULT) {
-        if (m_mpiInfo->size == 1) {
-            switch (method) {
-                case escript::SO_DEFAULT:
-                case escript::SO_METHOD_BICGSTAB:
-                case escript::SO_METHOD_CGLS:
-                case escript::SO_METHOD_GMRES:
-                case escript::SO_METHOD_LSQR:
-                case escript::SO_METHOD_PCG:
-                case escript::SO_METHOD_PRES20:
-                    package = escript::SO_PACKAGE_CUSP;
-                    break;
-                default:
-                    package = escript::SO_DEFAULT;
-                    break;
-            }
-            if (package == escript::SO_PACKAGE_CUSP) {
-                if (sb.getPreconditioner() != escript::SO_PRECONDITIONER_NONE &&
-                        sb.getPreconditioner() != escript::SO_PRECONDITIONER_JACOBI) {
-                    package = escript::SO_DEFAULT;
-                }
-            }
-        }
-    }
-#endif // ESYS_HAVE_CUDA
-
     // the configuration of ripley should have taken care that we have either
     // paso or trilinos so here's how we prioritize
 #if defined(ESYS_HAVE_PASO) && defined(ESYS_HAVE_TRILINOS)
@@ -913,18 +899,7 @@ int RipleyDomain::getSystemMatrixTypeId(const bp::object& options) const
     if (package == escript::SO_DEFAULT)
         package = escript::SO_PACKAGE_TRILINOS;
 #endif
-    if (package == escript::SO_PACKAGE_CUSP) {
-        if (m_mpiInfo->size > 1) {
-            throw NotImplementedError("CUSP matrices are not supported with more than one rank");
-        }
-        if (sb.isComplex()) {
-            throw NotImplementedError("CUSP does not support complex-valued matrices");
-        }
-        int type = (int)SMT_CUSP;
-        if (sb.isSymmetric())
-            type |= (int)SMT_SYMMETRIC;
-        return type;
-    } else if (package == escript::SO_PACKAGE_TRILINOS) {
+    if (package == escript::SO_PACKAGE_TRILINOS) {
 #ifdef ESYS_HAVE_TRILINOS
         int type = (int)SMT_TRILINOS;
         if (sb.isComplex())
@@ -999,16 +974,8 @@ escript::ASM_ptr RipleyDomain::newSystemMatrix(int row_blocksize,
     //    throw NotImplementedError("newSystemMatrix: reduced order not supported");
 
     if (type & (int)SMT_CUSP) {
-#ifdef ESYS_HAVE_CUDA
-        const dim_t numMatrixRows = getNumDOF();
-        bool symmetric = (type & (int)SMT_SYMMETRIC);
-        escript::ASM_ptr sm(new SystemMatrix(m_mpiInfo, row_blocksize,
-                    row_functionspace, numMatrixRows,
-                    getDiagonalIndices(symmetric), symmetric));
-        return sm;
-#else
-        throw RipleyException("newSystemMatrix: ripley was not compiled with "
-               "CUDA support so CUSP solvers & matrices are not available.");
+#ifndef ESYS_HAVE_CUDA
+        throw RipleyException("eScript does not support CUDA.");
 #endif
     } else if (type & (int)SMT_TRILINOS) {
 #ifdef ESYS_HAVE_TRILINOS
@@ -1364,7 +1331,7 @@ void RipleyDomain::updateTagsInUse(int fsType) const
 }
 
 #ifdef ESYS_HAVE_PASO
-void RipleyDomain::createPasoConnector(const RankVector& neighbour, 
+void RipleyDomain::createPasoConnector(const RankVector& neighbour,
                                        const IndexVector& offsetInSharedSend,
                                        const IndexVector& offsetInSharedRecv,
                                        const IndexVector& sendShared,
@@ -1641,7 +1608,7 @@ void RipleyDomain::assemblePDE(escript::AbstractSystemMatrix* mat,
     if (fsTypes.empty()) {
         return;
     }
-    
+
     int fs=fsTypes[0];
     if (fs != Elements && fs != ReducedElements)
         throw ValueError("assemblePDE: illegal function space type for coefficients");
@@ -1858,4 +1825,3 @@ void RipleyDomain::addPoints(const vector<double>& coords,
 }
 
 } // end of namespace ripley
-
