@@ -1,7 +1,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003-2018 by The University of Queensland
+# Copyright (c) 2003-2020 by The University of Queensland
 # http://www.uq.edu.au
 #
 # Primary Business: Queensland, Australia
@@ -11,6 +11,7 @@
 # Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 # Development 2012-2013 by School of Earth Sciences
 # Development from 2014 by Centre for Geoscience Computing (GeoComp)
+# Development from 2019 by School of Earth and Environmental Sciences
 #
 ##############################################################################
 
@@ -20,4 +21,10 @@
 from scons.templates.sid_py3_options import *
 
 mpi = 'OPENMPI'
+import sysconfig
+multiarch = sysconfig.get_config_var('MULTIARCH')
+mpi_include = '/usr/lib/' + multiarch + '/openmpi/include'
+mpi_lib = '/usr/lib/' + multiarch + '/openmpi/lib'
+mpi_prefix = [mpi_include,mpi_lib]
 
+cxx_extra=' -Wno-stringop-truncation'
\ No newline at end of file
