@@ -1,7 +1,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003-2018 by The University of Queensland
+# Copyright (c) 2003-2019 by the University of Queensland
 # http://www.uq.edu.au
 #
 # Primary Business: Queensland, Australia
@@ -11,12 +11,13 @@
 # Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 # Development 2012-2013 by School of Earth Sciences
 # Development from 2014 by Centre for Geoscience Computing (GeoComp)
+# Development from 2019 by School of Earth and Environmental Sciences
 #
 ##############################################################################
 
 from __future__ import print_function, division
 
-__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
+__copyright__="""Copyright (c) 2003-2019 by the University of Queensland
 http://www.uq.edu.au
 Primary Business: Queensland, Australia"""
 __license__="""Licensed under the Apache License, version 2.0
@@ -28,7 +29,7 @@ from distutils import sysconfig
 from subprocess import PIPE, Popen
 from SCons.Script.SConscript import Configure
 from site_init import findLibWithHeader, detectModule
-import subprocess 
+import subprocess
 
 REQUIRED_BOOST = (1, 46)
 
@@ -73,35 +74,50 @@ def checkCompiler(env):
 
     if not conf.CheckComplexAcos():
         conf.env.Append(CPPDEFINES = ['ESYS_USE_BOOST_ACOS'])
-    
+
     return conf.Finish()
 
-def get_external_python_sympy(bin):
+def get_external_python_sympy(env,bin):
     import subprocess
     cmd=''
     cmd+='import sympy\n'
     cmd+='print(sympy.__version__)\n'
     sp=subprocess.Popen([bin, '-c', cmd], stdin=None, stderr=None, stdout=subprocess.PIPE)
     #ver=sp.stdout.readline().strip().split('.')
-    
+
     import sys
     if sys.version_info[0] >= 3:
         ver = str(sp.stdout.readline().strip(), 'utf-8')
     else:
         ver = sp.stdout.readline().strip().split('.')
 
-    if int(ver[0]) == 0 and int(ver[1]) < 7:
+    tmp1 = ver[0]
+    tmp2 = ver[1]
+    if tmp2 == '.':
+        tmp2 = ver[2]
+        
+    if int(tmp1) == 0 and int(tmp2) < 7:
         env['sympy'] = False
         env['warnings'].append("sympy version is too old.")
+    if (int(tmp1) == 1 and int(tmp2) >= 2) or int(tmp1) >= 2:
+        env['sympy']=False
+        env['warnings'].append("escript does not support sympy version 1.2 and higher. Found version %d.%d" % (int(tmp1),int(tmp2)))
+
+    return env
 
 def call_python_config(bin=None):
     import subprocess
     cmd=''
     cmd+='import subprocess\n'
     cmd+='import os\n'
+    cmd+='import sys\n'
     cmd+='from distutils import sysconfig\n'
     cmd+='pyversion=sysconfig.get_python_version()\n'
-    cmd+='sp=subprocess.Popen(["python"+pyversion+"-config","--ldflags"], stdout=subprocess.PIPE)\n'
+    cmd+='try:\n'
+    cmd+='  sp=subprocess.Popen(["python"+pyversion+"-config","--ldflags"], stdout=subprocess.PIPE)\n'
+    cmd+='except:\n'
+    cmd+='  pythonroot=sys.exec_prefix+"/bin/"\n'
+    cmd+='  sp=subprocess.Popen([pythonroot+"python"+pyversion+"-config","--ldflags"], stdout=subprocess.PIPE)\n'
     cmd+='d=sp.stdout.readline().split()\n'
     cmd+="libdirs=[z[2:] for z in d if z.startswith(b'-L')]\n"
     cmd+="libs=[z[2:] for z in d if z.startswith(b'-lpython')]\n"
@@ -165,16 +181,19 @@ def checkPython(env):
     # Note: we assume scons is running python 2 in the following.
     else:
         (python_lib_path, python_libs,verstring, python_inc_path)=call_python_config(env['pythoncmd'])
-    
+
     if sys.version_info[0] == 3:
         if isinstance(verstring, str) is False:
             verstring = str(verstring, 'utf-8')
     else:
         if isinstance(verstring, basestring) is False:
             verstring = str(verstring, 'utf-8')
-            
+
     env['python_version'] = verstring
-    ispython3 = (verstring[0] == '3')
+    try:
+        ispython3 = (verstring[0] == '3')
+    except:
+        ispython3 = sys.version_info[0] == 3
     if ispython3:
         env.Append(CPPDEFINES=['ESPYTHON3'])
     env['buildvars']['python_version'] = verstring
@@ -262,70 +281,72 @@ def checkBoost(env):
     env['buildvars']['boostversion']=boostversion
 
     # Check for the boost numpy library
-    if boostversion >= 106300:
-        boost_numpy_inc_path,boost_numpy_lib_path=findLibWithHeader(env, env['boost_libs'], 'boost/python/numpy.hpp', env['boost_prefix'], lang='c++')
+    env['have_boost_numpy']=False
+    if boostversion >= 106300 and env['disable_boost_numpy'] is False:
+        try:
+            boost_numpy_inc_path,boost_numpy_lib_path=findLibWithHeader(env, env['boost_libs'], 'boost/python/numpy.hpp', env['boost_prefix'], lang='c++')
 
-        # Locate the boost numpy files
-        p = subprocess.Popen(["ld","--verbose"], stdout=subprocess.PIPE)
-        out,err = p.communicate()
-        spath = [x[13:-3] for x in out.split() if b'SEARCH_DIR' in x]
-        spath.append(boost_lib_path)
-        p2name = ''
-        p3name = ''
-        for name in spath:
-            try:
-                l=os.listdir(name)
-                
-                import sys
-                if sys.version_info[0] == 3:
-                    string_type = str
-                else:
-                    string_type = basestring
-                
-                p2res = ''
-                p3res = ''
-                for x in l:
-                    if isinstance(x,string_type):
-                        if x.startswith('libboost_numpy-py') and x.endswith('.so'):
-                            p2res = x
-                        if x.startswith('libboost_numpy-py3') and x.endswith('.so'):
-                            p3res = x
+            # Locate the boost numpy files
+            p = subprocess.Popen(["ld","--verbose"], stdout=subprocess.PIPE)
+            out,err = p.communicate()
+            spath = [x[13:-3] for x in out.split() if b'SEARCH_DIR' in x]
+            spath.append(boost_lib_path)
+            p2name = ''
+            p3name = ''
+            for name in spath:
+                try:
+                    l=os.listdir(name)
+
+                    import sys
+                    if sys.version_info[0] == 3:
+                        string_type = str
                     else:
-                        if x.startswith(b'libboost_numpy-py') and x.endswith(b'.so'):
-                            p2res = x
-                        if x.startswith(b'libboost_numpy-py3') and x.endswith(b'.so'):
-                            p3res = x
-                        
-                #p2res=[x for x in l if x.startswith('libboost_numpy-py') and x.endswith('.so')]
-                #p3res=[x for x in l if x.startswith('libboost_numpy3-py') and x.endswith('.so')]
+                        string_type = basestring
 
-                if len(p2name)==0 and len(p2res)>0:
-                    p2name=p2res[-1]
-                if len(p3name)==0 and len(p3res)>0:
-                    p3name=p3res[-1]
-            except OSError:
-                pass
+                    p2res = ''
+                    p3res = ''
+                    for x in l:
+                        if isinstance(x,string_type):
+                            if (x.startswith('libboost_numpy2')) and x.endswith('.so'):
+                                p2res = x
+                            if (x.startswith('libboost_numpy3')) and x.endswith('.so'):
+                                p3res = x
+                        else:
+                            if (x.startswith(b'libboost_numpy2')) and x.endswith(b'.so'):
+                                p2res = x
+                            if (x.startswith(b'libboost_numpy3')) and x.endswith(b'.so'):
+                                p3res = x
+                    if len(p2name)==0 and len(p2res)>0:
+                        p2name=p2res[3:-3]
+                    if len(p3name)==0 and len(p3res)>0:
+                        p3name=p3res[3:-3]
+                except OSError:
+                    pass
 
-        # Pick the right one
-        if int(env['python_version'][0]) == 2:
-            libname = p2name[3:-3]
-        else:
-            libname = p3name[3:-3]
+            # Pick the right one
+            if int(env['python_version'][0]) == 2:
+                libname = p2name
+            else:
+                libname = p3name
 
-        # If found, add the necessary information to env
-        if len(libname) > 0:
-            env.AppendUnique(LIBS = libname)
-            env['boost_libs'].append(libname)
-            env.AppendUnique(CPPPATH = [boost_numpy_inc_path])
-            env.AppendUnique(LIBPATH = [boost_numpy_lib_path])
-            env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], boost_numpy_lib_path)
-            env.Append(CPPDEFINES=['ESYS_HAVE_BOOST_NUMPY'])
+            # If found, add the necessary information to env
+            if len(libname) > 0:
+                env.AppendUnique(LIBS = libname)
+                tmp=env['boost_libs']
+                env['boost_libs']=[tmp,libname]
+                env.AppendUnique(CPPPATH = [boost_numpy_inc_path])
+                env.AppendUnique(LIBPATH = [boost_numpy_lib_path])
+                env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], boost_numpy_lib_path)
+                env.Append(CPPDEFINES=['ESYS_HAVE_BOOST_NUMPY'])
+                env['have_boost_numpy']=True
 
-    # boost_numpy_inc_path,boost_numpy_lib_path=findLibWithHeader(env, env['boost_libs'], 'boost/python/numpy.hpp', env['boost_prefix'], lang='c++')
-    # env.AppendUnique(CPPPATH = [boost_numpy_inc_path])
-    # env.AppendUnique(LIBPATH = [boost_numpy_lib_path])
-    # env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], boost_numpy_lib_path)
-    # env.Append(CPPDEFINES=['ESYS_HAVE_BOOST_NUMPY'])
+            print("Found boost/python/numpy.hpp. Building with boost numpy support.")
+        except:
+            print("Warning: Could not find boost/python/numpy.hpp. Building without numpy support.")
+
+    # Check if the version of boost we are using is missing BOOST_BYTE_ORDER
+    if boostversion >= 107000:
+        env.Append(CPPDEFINES=['ESYS_DEPRECATED_BOOST_ENDIAN'])
 
     return env
 
@@ -385,14 +406,18 @@ def checkOptionalModules(env):
         env['warnings'].append("Cannot import sympy. Symbolic toolbox and nonlinear PDEs will not be available.")
     else:
         if env['pythoncmd'] is not None:
-            get_external_python_sympy(env['pythoncmd'])
+            env=get_external_python_sympy(env, env['pythoncmd'])
         else:
             import sympy as sp
+            import distutils.version as duv
             spVer=sp.__version__
             spl=spVer.split('.')
-            if int(spl[0]) == 0 and int(spl[1]) < 7:
+            if duv.LooseVersion(sympy.__version__) < duv.LooseVersion('0.7'):
                 env['sympy']=False
                 env['warnings'].append("sympy version too old. Symbolic toolbox and nonlinear PDEs will not be available.")
+            if duv.LooseVersion(sympy.__version__) > duv.LooseVersion('1.2'):
+                env['sympy']=False
+                env['warnings'].append("escript does not support sympy version 1.2 and higher. Found %d" % duv.LooseVersion(sympy.__version__))
 
     ######## gmshpy
     env['gmshpy'] = detectModule(env, 'gmshpy')
@@ -405,8 +430,72 @@ def checkForTrilinos(env):
     if env['trilinos']:
         havelibs = (len(env['trilinos_libs']) > 0)
         trilinos_inc_path,trilinos_lib_path=findLibWithHeader(env,
-                env['trilinos_libs'], 'Tpetra_CrsMatrix.hpp',
+                env['trilinos_libs'], 'Amesos2.hpp',
                 env['trilinos_prefix'], lang='c++', try_link=havelibs)
+        env.AppendUnique(CPPPATH = [trilinos_inc_path])
+        env.AppendUnique(LIBPATH = [trilinos_lib_path])
+        env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], trilinos_lib_path)
+        env['buildvars']['trilinos_inc_path']=trilinos_inc_path
+        env['buildvars']['trilinos_lib_path']=trilinos_lib_path
+        conf = Configure(env.Clone())
+
+        dependencies=['Amesos2.hpp','Amesos2_Solver_decl.hpp','BelosSolverFactory.hpp','BelosSolverManager.hpp',\
+        'BelosTFQMRIter.hpp','BelosTFQMRSolMgr.hpp','BelosTpetraAdapter.hpp','BelosTypes.hpp',\
+        'Ifpack2_Factory.hpp','Kokkos_DefaultNode.hpp',\
+        'MatrixMarket_Tpetra.hpp','MueLu_CreateTpetraPreconditioner.hpp',\
+        'Teuchos_DefaultComm.hpp','Teuchos_ParameterList.hpp',\
+        'Tpetra_CrsGraph.hpp','Tpetra_CrsMatrix.hpp', 'Tpetra_RowMatrix.hpp',\
+        'Tpetra_Vector.hpp','Trilinos_version.h']
+
+        print("Looking for the Trilinos headers...")
+        for check in dependencies:
+            print("Checking for %s... %s" % (check, "yes" if os.path.isfile(os.path.join(trilinos_inc_path,check)) else "no"))
+            if not os.path.isfile(os.path.join(trilinos_inc_path,check)):
+                print("Could not find a Trilinos header file (tried looking in directory %s)" % (trilinos_inc_path))
+                env.Exit(1)
+
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_DefaultPlatform.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+            env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_DP'])
+
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_Experimental_BlockCrsMatrix.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+            env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKCRS'])
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        else:
+            raise RuntimeError('Could not locate the Trilinos Block CRS Matrix header')
+
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix_Helpers.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_Experimental_BlockCrsMatrix_Helpers.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+            env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKCRSH'])
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix_Helpers.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix_Helpers.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        else:
+            raise RuntimeError('Could not locate the Trilinos Block CRS Matrix Helpers header')
+
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockVector.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_Experimental_BlockVector.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+            env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKV'])
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockVector.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockVector.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        else:
+            raise RuntimeError('Could not locate the Trilinos BlockVector header')
+
+        # Try to extract the trilinos version from Trilinos_version.h
+        versionh=open(os.path.join(trilinos_inc_path, 'Trilinos_version.h'))
+        trilinos_version='unknown'
+        env['trilinos_version']='unknown'
+        for line in versionh:
+            ver=re.match(r'#define TRILINOS_MAJOR_MINOR_VERSION (\d+)',line)
+            if ver:
+                trilinos_version=ver.group(1)
+                trilinos_version = int(trilinos_version)
+                major=int(str(trilinos_version)[:2])
+                minor=int(str(trilinos_version)[2:4])
+                tmp=int(str(trilinos_version)[4:6])
+                env['trilinos_version'] = str(major)+"."+str(minor)+"."+str(tmp)
+
         if not havelibs:
             packages=['Tpetra','Kokkos','Belos','Amesos2','Ifpack2','MueLu']
             libs = []
@@ -428,12 +517,10 @@ def checkForTrilinos(env):
                     raise RuntimeError('Error reading Trilinos export Makefile\n%s'%(e))
             env['trilinos_libs'] = libs
 
-        env.AppendUnique(CPPPATH = [trilinos_inc_path])
-        env.AppendUnique(LIBPATH = [trilinos_lib_path])
         env.Append(CPPDEFINES = ['ESYS_HAVE_TRILINOS'])
-        env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], trilinos_lib_path)
-        env['buildvars']['trilinos_inc_path']=trilinos_inc_path
-        env['buildvars']['trilinos_lib_path']=trilinos_lib_path
+        # env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], trilinos_lib_path)
+        # env['buildvars']['trilinos_inc_path']=trilinos_inc_path
+        # env['buildvars']['trilinos_lib_path']=trilinos_lib_path
     env['buildvars']['trilinos']=int(env['trilinos'])
     return env
 
@@ -455,19 +542,6 @@ def checkOptionalLibraries(env):
         env['buildvars']['netcdf_lib_path']=netcdf_lib_path
     env['buildvars']['netcdf']=int(env['netcdf'])
 
-    ######## PAPI
-    papi_inc_path=''
-    papi_lib_path=''
-    if env['papi']:
-        papi_inc_path,papi_lib_path=findLibWithHeader(env, env['papi_libs'], 'papi.h', env['papi_prefix'], lang='c++')
-        env.AppendUnique(CPPPATH = [papi_inc_path])
-        env.AppendUnique(LIBPATH = [papi_lib_path])
-        env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], papi_lib_path)
-        env.Append(CPPDEFINES = ['ESYS_HAVE_PAPI'])
-        env['buildvars']['papi_inc_path']=papi_inc_path
-        env['buildvars']['papi_lib_path']=papi_lib_path
-    env['buildvars']['papi']=int(env['papi'])
-
     ######## MKL
     mkl_inc_path=''
     mkl_lib_path=''
@@ -568,7 +642,7 @@ def checkOptionalLibraries(env):
         env['buildvars']['visit_inc_path']=visit_inc_path
         env['buildvars']['visit_lib_path']=visit_lib_path
     env['buildvars']['visit']=int(env['visit'])
-    
+
     ######## MPI
     if env['mpi']=='no':
         env['mpi']='none'
@@ -657,35 +731,36 @@ def checkOptionalLibraries(env):
 
     ######## gmsh (for tests)
     env['gmsh'] = False
-    if env['IS_WINDOWS']:
-        try:
-            p=Popen(['gmsh', '-info'], stderr=PIPE)
-            _,e=p.communicate()
-            if e.split().count("MPI"):
-                env['gmsh']='m'
-            else:
-                env['gmsh']='s'
-        except OSError:
-            pass
-    else:
-        which = Popen(['which', 'gmsh'], stdout=PIPE)
-        path,_ = which.communicate()
-        if which.wait() == 0:
-            cmd = ['ldd', path[:-1]]
-            if env['IS_OSX']:
-                cmd = ['otool','-L', path[:-1]]
+    if env['use_gmsh'] is True:
+        if env['IS_WINDOWS']:
             try:
-                p=Popen(cmd, stdout=PIPE)
-                gmshlibs,_ = p.communicate()
-                env.Append(CPPDEFINES=['ESYS_HAVE_GMSH'])
-                if p.returncode == 0 and 'libmpi' in gmshlibs:
-                    env['gmsh'] = 'm'
-                    env.Append(CPPDEFINES=['ESYS_GMSH_MPI'])
+                p=Popen(['gmsh', '-info'], stderr=PIPE)
+                _,e=p.communicate()
+                if e.split().count("MPI"):
+                    env['gmsh']='m'
                 else:
-                    env['gmsh'] = 's'
+                    env['gmsh']='s'
             except OSError:
                 pass
-    
+        else:
+            which = Popen(['which', 'gmsh'], stdout=PIPE)
+            path,_ = which.communicate()
+            if which.wait() == 0:
+                cmd = ['ldd', path[:-1]]
+                if env['IS_OSX']:
+                    cmd = ['otool','-L', path[:-1]]
+                try:
+                    p=Popen(cmd, stdout=PIPE)
+                    gmshlibs,_ = p.communicate()
+                    env.Append(CPPDEFINES=['ESYS_HAVE_GMSH'])
+                    if p.returncode == 0 and 'libmpi' in gmshlibs:
+                        env['gmsh'] = 'm'
+                        env.Append(CPPDEFINES=['ESYS_GMSH_MPI'])
+                    else:
+                        env['gmsh'] = 's'
+                except OSError:
+                    pass
+
     ######## boost::iostreams
     if env['compressed_files']:
         try:
@@ -705,5 +780,3 @@ def checkPDFLatex(env):
     else:
         env['pdflatex']=False
     return env
-
-
